@@ -628,7 +628,7 @@ static void CheckDropPrivsMustBeDisabled(pkgAcquire const &Fetcher)
       // if destination file is inaccessible all hope is lost for privilege dropping
       if (IsAccessibleBySandboxUser((*I)->DestFile, true) == false)
       {
-	 _error->WarningE("pkgAcquire::Run", _("Can't drop privileges for downloading as file '%s' couldn't be accessed by user '%s'."),
+	 _error->WarningE("pkgAcquire::Run", _("Download is performed unsandboxed as root as file '%s' couldn't be accessed by user '%s'."),
 	       (*I)->DestFile.c_str(), SandboxUser.c_str());
 	 _config->Set("APT::Sandbox::User", "");
 	 break;
@@ -645,7 +645,7 @@ static void CheckDropPrivsMustBeDisabled(pkgAcquire const &Fetcher)
 
 	 if (IsAccessibleBySandboxUser(source.Path, false) == false)
 	 {
-	    _error->NoticeE("pkgAcquire::Run", _("Can't drop privileges for downloading as file '%s' couldn't be accessed by user '%s'."),
+	    _error->NoticeE("pkgAcquire::Run", _("Download is performed unsandboxed as root as file '%s' couldn't be accessed by user '%s'."),
 		  source.Path.c_str(), SandboxUser.c_str());
 	    _config->CndSet("Binary::file::APT::Sandbox::User", "root");
 	    _config->CndSet("Binary::copy::APT::Sandbox::User", "root");
